<?php

class BL_Auth_Manager_Users
{
	const CREATE = 1;
	const UPDATE = 2;
	const REMOVE = 3;

	private static $instance = NULL;
	/**
	 *
	 * @var BL_Context
	 */
	protected $context;
	/**
	 *
	 * @var BL_Database
	 */
	protected $db;

	/*
		'login',
		'mail',
		'pass',
		'date',
		'is_admin',
		'is_active'
	 */
	protected $data = array();
	
	protected $errors = array();

	protected $settings = array();

	// additional rules
	protected $rules = array();

	protected $messages = array(
		'userNotFound' => 'User with %identName% = %identValue% not found'
	);

	/**
	 *
	 * @var BL_Auth_Validator_User
	 */
	protected $avu;

	protected function __construct(BL_Context $context)
	{
		$this->context = $context;
		$this->db = $context->db();
		$this->settings = BL_Settings::getMerged($this->getDefaultSettings(), 'auth.users.manager');

		$this->avu = new BL_Auth_Validator_User($this->settings);
		BL_Validator::attachValidator($this->avu);
	}
	
	protected function getDefaultSettings()
	{
		return array(
			//  ,       is_admin     root'
			'onlyRootCanAddAdmin' => true, #   
			'users'  => array(
				'withoutPass' => false, #   
				'withoutMail' => false, #   
				'withoutNick' => false,
				'isActive'    => false #   
			),
			'admins' => array(
				'withoutPass' => false, #   
				'withoutNick' => false, 
				'isActive'    => false  #   
			)
		);
	}

	/**
	 *
	 * @param array $settings
	 * @return BL_Auth_Manager_Users
	 */
	public function setSettings(array $settings)
	{
		$this->settings = array_merge_recursive_distinct($this->settings, $settings);
		$this->avu->setSettings($this->settings);
		return $this;
	}

	/**
	 *
	 * @param array $messages
	 * @return BL_Auth_Manager_Users
	 */
	public function setMessages(array $messages)
	{
		$this->messages = array_merge_recursive_distinct($this->messages, $messages);
		$this->avu->setMessages($this->messages);
		return $this;
	}

	/**
	 *
	 * @return BL_Auth_Manager_Users
	 */
	public static function instance(BL_Context $context = NULL)
	{
		return self::$instance ? self::$instance : self::$instance = new self($context);
	}
	
	public static function generateSalt($length = 3)
	{
		for ($i = 0, $salt = ''; $i < $length; $i++) {
			$salt .= chr(rand(33, 126));
		}
		return $salt;
	}
	
	public static function generateHash()
	{
		return md5(rand(0, 127) . microtime(true));
	}

	public function assign($field, $value = NULL)
	{
		if (is_array($field)) {
			foreach ($field as $key => $value)
				$this->assign($key, $value);
		} else {
			$this->data[$field] = $value;
		}
		
		return $this;
	}

	public function addRule($action, $field, $rule, $messages = array(), $options = array())
	{
		if (!isset($this->rules[$action]))
				$this->rules[$action] = array();
		$this->rules[$action][] = array($field, $rule, $messages, $options);

		return $this;
	}

	public function validate($action, $id = 0)
	{
		$validator = BL_Validator::instance(__CLASS__ . $action);

		$validator
			->addRule('login', 'noSkipOnEmpty')
			->addRule('mail', 'noSkipOnEmpty');

		switch ($action) {
			case self::CREATE:
				$validator
					->addRule('login', 'user.login')
					->addRule('mail', 'user.mail');
				break;

			case self::UPDATE:
				$validator
					->addRule('login', 'user.login', array(), $id)
					->addRule('mail', 'user.mail', array(), $id);
				break;
		}

		$validator
			->addRule('pass', 'noSkipOnEmpty')
			->addRule('pass', 'user.pass')

			->addRule('date', 'noSkipOnEmpty')
			->addRule('date', 'user.date')

			->addRule('is_admin', 'noSkipOnEmpty')
			->addRule('is_admin', 'user.is_admin')

			->addRule('is_active', 'noSkipOnEmpty')
			->addRule('is_active', 'user.is_active');

		if (isset($this->rules[$action])) {
			foreach ($this->rules[$action] as $rulesData) {
				$validator->addRule($rulesData[0], $rulesData[1], $rulesData[2], $rulesData[3]);
			}
		}
		//TODO:  validData  2 ?
		return $validator->validate($this->data, $this->data, $this->errors);
	}
	
	public function getLastErrors()
	{
		return $this->errors;
	}

	public function getLastError($field)
	{
		return isset($this->errors[$field]) ? $this->errors[$field] : NULL;
	}

	public function create()
	{
		//TODO:       2 
		if (!$this->validate(self::CREATE))
			return false;
		
		$data = array_merge(
			$this->preparePass($this->data['pass']),
			array(
				'date'      => $this->data['date'],
				'is_admin'  => $this->data['is_admin'],
				'is_active' => $this->data['is_active'],
			)
		);
		
		$this->data['login'] && $data['login'] = $this->data['login'];
		$this->data['mail']  && $data['mail']  = $this->data['mail'];

		$id = $this->db->insert('INSERT INTO :users SET ?@', $data);
		$pass = $this->data['pass'];
		$this->data = array();

		return array_merge(array('id' => $id, 'pass' => $pass), $data);
	}
	
	public function update($ident, $key = 'auto')
	{
		if (!$id = $this->checkExists($ident, $key) || !$this->validate(self::UPDATE, $id))
			return false;
		
		$data = array(
			'date'      => $this->data['date'],
			'is_admin'  => $this->data['is_admin'],
			'is_active' => $this->data['is_active'], //        
		);
		//     -      
		$this->data['pass'] && $data = array_merge($data, $this->preparePass($this->data['pass']));

		$this->data['login'] && $data['login'] = $this->data['login'];
		$this->data['mail']  && $data['mail']  = $this->data['mail'];
		
		$this->db->query('UPDATE :users SET ?@ WHERE id=?i', $data, $id);
		$pass = $this->data['pass'];
		$this->data = array();

		return array_merge(array('id' => $id, 'pass' => $pass), $data);
	}

	//TODO: GC (delete old users without activation)
	public function rawUpdate($userId, $data)
	{
		$this->db->query('UPDATE :users SET ?@ WHERE id=?i', $data, $userId);
		return $this;
	}

	protected function checkExists($ident, $key)
	{
		if ($id = $this->context->users()->getBaseInfo($ident, $key, 'id')) {
			return $id;
		}
		// store error
		$this->errors = array($key => str_replace(array('%identName%', '%identValue%'), array($key, $ident), $this->messages['userNotFound']));
		return false;
	}

	public function remove($ident, $key = 'auto')
	{
		if (!$id = $this->checkExists($ident, $key))
			return false;
		
		$this->db->query('DELETE FROM :users WHERE id=?i', $id);
		//TODO: notify event
		return true;
	}
	
	public function preparePass($pass)
	{
		$salt = self::generateSalt();
		$pass = md5(md5($pass) . $salt);
		
		return array('salt' => $salt, 'passhash' => $pass);
	}
}
