<?php

abstract class BL_Auth_Manager_UsersActionsAbstract
{
	const SUCCESS      = 1;
	const NOT_FOUND    = 2;
	const ANOTHER_TYPE = 3;
	const TOO_OLD      = 4;

	/**
	 *
	 * @var BL_Database
	 */
	protected $db;

	protected $lifeTime = 86400;

	public function __construct()
	{
		$this->db = BL_Context::instance()->db();
	}

	abstract public function getTypeId();

	public function lifeTime($time = 0)
	{
		if ($time > 0) {
			$this->lifeTime = $time;
			return $this;
		}
		return $this->lifeTime;
	}

	public function write($userId, $data = '')
	{
		$action = array(
			'user_id' => $userId,
			'hash'    => BL_Auth_Manager_Users::generateHash(),
			'type'    => $this->getTypeId(),
			'date'    => date(DATE_MYSQL)
		);

		if (is_scalar($data)) {
			$action['data'] = $data;
		} else {
			$action['data'] = serialize($data);
			$action['serialized'] = 1;
		}

		$this->db->query('
			INSERT INTO :users_actions SET ?@ ON DUPLICATE KEY UPDATE ?@', $action, $action);

		return $action['hash'];
	}

	public function read($userId, $hash, &$code = NULL)
	{
		$action = $this->db->fetch('
			SELECT * FROM :users_actions WHERE user_id=?i AND hash=?s', $userId, $hash);

		if (!$action) {
			$code = self::NOT_FOUND;
			return false;
		}

		if ($this->getTypeId() != $action['type']) {
			$code = self::ANOTHER_TYPE;
			return false;
		}

		if (time() - strtotime($action['date']) > $this->lifeTime()) {
			$code = self::TOO_OLD;
			return false;
		}
		
		$code = self::SUCCESS;
		return $action['serialized'] ? unserialize($action['data']) : $action['data'];
	}

	public function delete($userId)
	{
		$this->db->query('
			DELETE FROM :users_actions WHERE user_id=?i AND type=?i', $userId, $this->getTypeId());

		return $this;
	}
}
