<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Auth_User
{
	const E_IDENT_EMPTY  = '_1';
	const E_PASS_EMPTY   = '_2';
	const E_ALREADY_AUTH = '_3';
	const E_USER_NOT_FOUND     = '_4';
	const E_USER_NOT_ACTIVATED = '_5';
	const E_USER_NOT_ADMIN     = '_6';
	const E_BAD_PASS = '_7';
	
	private static $instance = NULL;

	protected $db;

	protected $settings = array();
	
	protected $messages = array(
		self::E_IDENT_EMPTY   => 'Ident %identName% is empty',
		self::E_PASS_EMPTY    => 'Empty pass',
		self::E_ALREADY_AUTH  => 'User already authenticated, logout first, please',
		self::E_USER_NOT_FOUND     => 'User with ident %identName%=%identValue% isn\'t exists',
		self::E_USER_NOT_ACTIVATED => 'User with ident %identName%=%identValue% isn\'t activated',
		self::E_USER_NOT_ADMIN     => 'User with ident %identName%=%identValue% isn\'t admin',
		self::E_BAD_PASS => 'Password incorrect',
	);

	protected $errors = array();

	/**
	 *
	 * @var BL_Auth_Users
	 */
	protected $users;
	
	protected $userId   = NULL;
	protected $passHash = NULL;

	protected $needReCheckAuth = true;
	protected $isAuthenticated = false;

	protected function __construct(BL_Context $context)
	{
		$this->db    = $context->db();
		$this->users = $context->users();
		$this->settings = BL_Settings::getMerged($this->getDefaultSettings(), 'auth.user');

		$R = $context->request();
		$this->userId   = $R->cookies($this->settings['cookieNames']['id']);
		$this->passHash = $R->cookies($this->settings['cookieNames']['hash']);
	}

	protected function getDefaultSettings()
	{
		return array(
			'cookieNames'  => array(
				'id'   => 'id',
				'hash' => 'hash'
			),
			// ,    
			'lifeTime'    => 86400,
			//        isAuthenticated
			'prolongAuth' => true,
			'cookieSalt'  => 'sk!3w',
			//     
			'allowRelogin' => true
			//TODO:  hash  
			//TODO:   IP
		);
	}

	/**
	 *
	 * @return BL_Auth_User
	 */
	public static function instance(BL_Context $context = NULL)
	{
		return self::$instance ? self::$instance : self::$instance = new self($context);
	}

	/**
	 *
	 * @param array $messages
	 * @return BL_Auth_User
	 */
	public function setMessages(array $messages)
	{
		$this->messages = array_merge($this->messages, $messages);

		return $this;
	}
	
	protected function _isAuthenticated()
	{
		if ($this->needReCheckAuth) {
			if (!$this->userId || !$this->passHash) {
				$this->needReCheckAuth = false;
				return false;
			}

			$data = $this->users->getBaseInfo($this->userId, 'id');
	
			if (empty($data)) {
				$this->needReCheckAuth = false;
				return $this->isAuthenticated = false;
			}
			
			if (md5($data['passhash'] . $this->settings['cookieSalt']) == $this->passHash && $data['is_active']) {
				$this->isAuthenticated = true;
				if ($this->settings['prolongAuth'])
					$this->sendValidCookies($data['id'], $data['passhash']);
			}
			
			$this->needReCheckAuth = false;
		}
		
		return $this->isAuthenticated;
	}
	
	public static function isAuthenticated()
	{
		return self::instance()->_isAuthenticated();
	}
	
	public static function isAdmin()
	{
		if (self::isAuthenticated()) {
			return (bool)self::instance()->users->getBaseInfo(self::instance()->userId, 'id', 'is_admin');
		}
		return false;
	}
	
	public static function isRoot()
	{
		// if user_id == 1 -> root
		return self::isAuthenticated() && 1 == self::instance()->userId;
	}
	
	public function silentLogIn($userId)
	{
		//TODO:   
		if ($data = $this->users->getBaseInfo($userId, 'id')) {
			$this->sendValidCookies($data['id'], $data['pass']);
			return true;
		}
		return false;
	}
	
	public function logIn($ident, $pass, $key = 'name', $asAdmin = false)
	{
		$ident = trim($ident);
		$pass  = trim($pass);
		$this->errors = array();
		
		if (empty($ident))
			$this->errors['ident'] = str_replace('%identName%', $key, $this->messages[self::E_IDENT_EMPTY]);
		
		if (empty($pass))
			$this->errors['pass'] = $this->messages[self::E_PASS_EMPTY];
		
		if ($this->errors)
			return false;
		
		if (!$this->settings['allowRelogin'] && $this->_isAuthenticated()) {
			$this->errors = array($this->messages[self::E_ALREADY_AUTH]);
			return false;
		}
		
		$data = $this->users->getBaseInfo($ident, $key);
		
		if (empty($data)) {
			$this->errors['ident'] = str_replace(array('%identName%', '%identValue%'), array($key, $ident), $this->messages[self::E_USER_NOT_FOUND]);
			return false;
		}
		
		if (!$data['is_active']) {
			$this->errors['ident'] = str_replace(array('%identName%', '%identValue%'), array($key, $ident), $this->messages[self::E_USER_NOT_ACTIVE]);
			return false;
		}
		//TODO:  .        
		if (!$data['is_admin'] && $asAdmin) {
			$this->errors['ident'] = str_replace(array('%identName%', '%identValue%'), array($key, $ident), $this->messages[self::E_USER_NOT_ADMIN]);
			return false;
		}
		//TODO:   -   
		if ($this->users->checkPass($data['id'], $pass)) {
			$this->sendValidCookies($data['id'], $data['passhash']);
			return true;
		} else {
			$this->errors['pass'] = str_replace(array('%identName%', '%identValue%'), array($key, $ident), $this->messages[self::E_BAD_PASS]);
			return false;
		}
	}

	public function getLastErrors()
	{
		return $this->errors;
	}

	public function getBaseInfo($field = NULL)
	{
		$data = array();
		if (self::isAuthenticated())
			$data = $this->users->getBaseInfo($this->userId, 'id');
		
		if ($field)
			return isset($data[$field]) ? $data[$field] : NULL;

		return $data;
	}

	protected function sendValidCookies($id, $pass)
	{
		$hash = md5($pass . $this->settings['cookieSalt']);
		$this->userId = $id;
		$this->passHash = $hash;

		BL_Response::instance()
			->setCookie($this->settings['cookieNames']['id'], $id, $this->settings['lifeTime'])
			->setCookie($this->settings['cookieNames']['hash'], $hash, $this->settings['lifeTime']);

		$this->isAuthenticated = true;
	}
	
	public function logOut()
	{
		$this->userId = NULL;
		$this->passHash = NULL;
		BL_Response::instance()
			->removeCookie($this->settings['cookieNames']['id'])
			->removeCookie($this->settings['cookieNames']['hash']);
		
		$this->isAuthenticated = false;
	}
}
