<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Auth_Users
{
	/**
	 *
	 * @var BL_Auth_Users
	 */
	private static $instance = NULL;

	/**
	 *
	 * @var BL_Database
	 */
	protected $db;

	/**
	 *
	 * @var BL_Context
	 */
	protected $context;

	protected function __construct(BL_Context $context)
	{
		$this->context = $context;
		$this->db = $context->db();
	}

	/**
	 *
	 * @return BL_Auth_Users
	 */
	public static function instance(BL_Context $context = NULL)
	{
		return self::$instance ? self::$instance : self::$instance = new self($context);
	}
	
	protected $userData = array(
		'id'   => array(),
		'name' => array(),
		'mail' => array()
	);
	//TODO: key as array
	public function getBaseInfo($value, $key = 'auto', $field = NULL)
	{
		$keys = array('id', 'login', 'mail', 'auto');

		if (!in_array($key, $keys))
			throw new BL_Auth_Exception('Invalid ident name');

		if ($key == 'auto') {
			if (strpos($value, '@') !== false) {
				$key = 'mail';
				$value = strtolower($value);
			} elseif ($value == (string)ceil($value)) {
				$key = 'id';
			} else {
				$key = 'login';
				$value = strtolower($value);
			}
		} else {
			$value = strtolower($value);
		}

		if (isset($this->userData[$key][$value]))
			return ($field && array_key_exists($field, $this->userData[$key][$value])) ?
				$this->userData[$key][$value][$field] : $this->userData[$key][$value];

		$data = $this->db->fetch('
			SELECT id, login, mail, passhash, salt, date, is_active, is_admin, need_permissions
			FROM :users WHERE ?c=?s', $key, $value);

		if ($data) {
			$ED = $this->context->eventDispatcher();
			$event = $ED->event($this, 'system.user.getInfo', array('id' => $data['id']));
			$data = $ED->filter($event, $data)->getReturnValue();
		}

		if ($data) {
			$this->userData['id'][$data['id']]     =
			$this->userData['login'][strtolower($data['login'])] =
			$this->userData['mail'][strtolower($data['mail'])] = $data;
		} else {
			$this->userData[$key][$value] = $data;
		}
		
		return ($field && isset($this->userData[$key][$value][$field])) ?
			$this->userData[$key][$value][$field] : $this->userData[$key][$value];
	}

	public function checkPass($userId, $pass)
	{
		$userInfo = $this->getBaseInfo($userId, 'id');
		return md5(md5($pass) . $userInfo['salt']) == $userInfo['passhash'];
	}
}
