<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Auth_Validator_User extends BL_Validator_Abstract
{
	protected $settings = array();

	protected $messages = array(
		'user.login' => array(
			'empty' => ' ',
			'used'  => ' %login%  '
		),
		'user.mail' => array(
			'empty' => ' email',
			'used'  => 'Email %mail%  '
		),
		'user.pass' => array(
			'empty' => ' '
		)
	);

	/**
	 *
	 * @var BL_Auth_Users
	 */
	protected $users;

	// ID of user wich edited
	protected $userId = 0;
	
	public function __construct(array $settings)
	{
		$this->settings = $settings;
		$this->users = BL_Context::instance()->users();
	}

	public function setSettings(array $settings)
	{
		$this->settings = $settings;
		return $this;
	}

	public function validate($rule, $field, &$value, &$error, array $messages = array(), array $options = array())
	{
	    $isAdmin = (bool)$this->validator->getValue('is_admin');
			$id = $this->getOption($options, 0, -1, array(0));
			$token = $isAdmin ? 'admins' : 'users';

	    switch ($rule) {
		    case 'user.login':
					if (!$value && !$this->settings[$token]['withoutNick']) {
						// store error
						$error = $this->getMessage($rule, $messages, 'empty');
						return false;
					}
					// user with login=$value already exist and it is not this user
					if (($_id = $this->users->getBaseInfo($value, 'login', 'id')) && $_id != $id) {
						// store error
						$error = $this->formatMessage($this->getMessage($rule, $messages, 'used'), array('login' => $value));
						return false;
					}
					break;

		    case 'user.mail':
					if (!$value && !$this->settings[$token]['withoutMail']) { //TODO: admins aleays need mail
						// store error
						$error = $this->getMessage($rule, $messages, 'empty');
						return false;
					}
					// user with mail=$value already exist and it is not this user
					if (($_id = $this->users->getBaseInfo($value, 'mail', 'id')) && $_id != $id) {
						// store error
						$error = $this->formatMessage($this->getMessage($rule, $messages, 'used'), array('mail' => $value));
						return false;
					}
				break;
			
			case 'user.date':
				//TODO: check format
				$value || $value = date(DATE_MYSQL);
				break;

			case 'user.pass':
				//TODO: use settings, generate pass
				if (!$value) {
					// store error
					$error = $this->getMessage($rule, $messages, 'empty');
					return false;
				}
				break;

			case 'user.is_admin':
				$value = $value ? 1 : 0;
				break;
			
			case 'user.is_active':
				$value = $value ? 1 : 0;
				break;
	   }
	   return true;
   }

	public function getRules()
	{
		return array('user.login', 'user.mail', 'user.pass', 'user.date', 'user.is_admin', 'user.is_active');
	}
}
