<?php

class BL_Context
{
	/**
	 *
	 * @var BL_Loader
	 */
	protected $loader;
	/**
	 *
	 * @var BL_Application
	 */
	protected $app;
	/**
	 *
	 * @var BL_Request
	 */
	protected $request;
	/**
	 *
	 * @var BL_Response
	 */
	protected $response;
	/**
	 *
	 * @var BL_Page
	 */
	protected $page;
	/**
	 *
	 * @var BL_Locale
	 */
	protected $locale;
	/**
	 *
	 * @var BL_Event_Dispatcher
	 */
	protected $eventDispatcher;

	protected $db = array();

	/**
	 *
	 * @var BL_Session
	 */
	protected $session = NULL;

	protected $user = NULL;

	protected $users = NULL;

	protected $usersManager = NULL;

	private static $instance = NULL;

	public function __construct()
	{
		$this->request  = BL_Request::instance(); //TODO: use `new`
		$this->response = BL_Response::instance($this->request);
		$this->page     = BL_Page::instance();
	}

	/**
	 *
	 * @return BL_Context
	 */
	public static function instance()
	{
		return self::$instance ? self::$instance : self::$instance = new self;
	}

	public function injectApp(BL_Application $app)
	{
		$this->app = $app;
		$eventsDir = 'config/application/';
		$eventsDir .= $app->isBackend() ? 'admin/events' : 'events';
		$this->eventDispatcher = new BL_Event_Dispatcher($eventsDir);

		$this->loader
			->injectApp($app)
			->injectContext($this);

		return $this;
	}

	public function injectLoader(BL_Loader $loader)
	{
		$this->loader = $loader;
	}
	/**
	 *
	 * @return BL_Application
	 */
	public function app()
	{
		return $this->app;
	}

	/**
	 *
	 * @return BL_Event_Dispatcher
	 */
	public function eventDispatcher()
	{
		return $this->eventDispatcher;
	}

	/**
	 *
	 * @return BL_Page
	 */
	public function page()
	{
		return $this->page;
	}

	/**
	 *
	 * @return BL_Request
	 */
	public function request()
	{
		return $this->request;
	}

	/**
	 *
	 * @return BL_Response
	 */
	public function response()
	{
		return $this->response;
	}

	/**
	 *
	 * @return BL_Database
	 */
	public function db($token = 'default')
	{
		return isset($this->db[$token]) ? $this->db[$token] : $this->db[$token] = new BL_Database(BL_Settings::get('database', $token));
	}
	/**
	 *
	 * @return BL_Loader
	 */
	public function loader()
	{
		return $this->loader;
	}

	/**
	 *
	 * @return BL_Session
	 */
	public function session()
	{
		return $this->session ? $this->session : $this->session = new BL_Session($this->request, $this->response, array(), $this->eventDispatcher);
	}
	/**
	 *
	 * @return BL_Auth_User
	 */
	public function user()
	{
		return $this->user ? $this->user : $this->user = BL_Auth_User::instance($this);
	}
	/**
	 *
	 * @return BL_Auth_Users
	 */
	public function users()
	{
		return $this->users ? $this->users : $this->users = BL_Auth_Users::instance($this);
	}

	/**
	 *
	 * @return BL_Auth_Manager_Users
	 */
	public function usersManager()
	{
		return $this->usersManager ? $this->usersManager : $this->usersManager = BL_Auth_Manager_Users::instance($this);
	}
}
