<?php
//TODO???: PDO
//TODO: setFetchMode
//TODO: FETCH_OBJ
//TODO: fetchPairs
class BL_Database
{
	/**
	 *
	 * @var mysqli
	 */
	protected $li = NULL;
	
	protected $debug = true;
	
	protected $settings = array();
	/**
	 *
	 * @var BL_Database_Placeholder
	 */
	protected $placeholder = NULL;
	
	//TODO???:  ,      BL_Database_Result class?
	//   
	protected $result = NULL;
	
	protected $resultType = MYSQLI_ASSOC;

	protected $transactions = array();

	// debug layer
	protected $queriesCount = 0;
	protected $debugInformation = array();
	
	protected function getDefaultSettings()
	{
		return array(
			'host'    => '127.0.0.1',
			'name'    => 'root',
			'pass'    => '',
			'dbname'  => 'database',
			'port'    => ini_get('mysqli.default_port'),
			'socket'  => ini_get('mysqli.default_socket'),
			'prefix'  => 'dt',
			'charset' => synonymEncoding(BL_APP_CHARSET),
			'debug'   => true,
			'lazy'    => true
		);
	}

	// NB! for internal using only
	public function getSettings()
	{
		return $this->settings;
	}

	protected function _connect()
	{
		$er = error_reporting(0);
		$this->li = new mysqli($this->settings['host'], $this->settings['name'], $this->settings['pass'], $this->settings['dbname'], $this->settings['port'], $this->settings['socket']);
		error_reporting($er);
       	
		if (mysqli_connect_errno()) {
			throw new BL_Database_Exception(mysqli_connect_error(), mysqli_connect_errno());
		}
		
		$this->li->query("SET NAMES '{$this->settings['charset']}'");
		$this->placeholder = new BL_Database_Placeholder($this->li, $this->settings['prefix']);
	}
	
	public function __construct(array $settings = array())
	{
		$this->settings = array_merge($this->getDefaultSettings(), $settings);
		
		$this->debug = (bool)$this->settings['debug'];
		
		if (!$this->settings['lazy']) {
			$this->_connect();
		}
	}

	/**
	 *
	 * @return mysqli
	 */
	public function getConnection()
	{
		return $this->li;
	}
	
	protected function _prepare($argv)
	{
		if (!$this->li) {
			$this->_connect();
		}
		
		return $this->placeholder->prepareQuery($argv);
	}
	
	protected function _query($argv, $multi = false)
	{
		$query = $this->_prepare($argv);
		$this->queriesCount++;
		
		if ($this->debug) {
			$this->debugInformation[$this->queriesCount]['query'] = $query;
			$ts = microtime(true);
		}
		
		$old = error_reporting(0);
		if ($multi) {
			//TODO:    
			$this->li->multi_query($query);
			$this->result = array();
			do {
				if ($result = $this->li->store_result()) {
					//$result->free();
					$this->result[] = $result;
				}
			} while ($this->li->next_result());
		} else {
			$this->result = $this->li->query($query, MYSQLI_STORE_RESULT);
		}
		error_reporting($old);
		
		if ($this->li->errno) {
			$msg = sprintf('Error in query. Query="%s" error="%s"', $query, $this->li->error);
			throw new BL_Database_Exception($msg, $this->li->errno);
		}
		
		if ($this->debug) {
			$this->debugInformation[$this->queriesCount]['time'] = microtime(true) - $ts;
		}
	}
	
	protected function _fetch(mysqli_result $result)
	{
		return $result->fetch_array($this->resultType);
	}
	
	protected function _fetchAll(mysqli_result $result)
	{
		$a = array();
		while ($row = $result->fetch_array($this->resultType))
			$a[] = $row;
		
		return $a;
	}
	
	protected function _getColumn(mysqli_result $result)
	{
		$n = $result->num_rows;
		for($i = 0; $i < $n; $i++) {
			$a = $result->fetch_array(MYSQLI_NUM);
			$b[] = $a[0];
		}
		return $b;
	}
	
	protected function _numRows(mysqli_result $result)
	{
		return $result->num_rows;
	}
	
	public function setResultType($type)
	{
		$old = $this->resultType;
		if (in_array($type, array(MYSQLI_ASSOC, MYSQLI_BOTH, MYSQLI_NUM)))
			$this->resultType = $type;
		
		return $old;
	}

	/**
	 *
	 * @param string $transactionId
	 * @return BL_Database_TransactionStorage
	 */
	public function transaction($transactionId)
	{
		return isset($this->transactions[$transactionId]) ? $this->transactions[$transactionId] :
			$this->transactions[$transactionId] = new BL_Database_TransactionStorage($this);
	}

	public function getTableName($table)
	{
		return $this->settings['prefix'] . $table;
	}

	public function getResultType()
	{
		return $this->resultType;
	}
	
	public function getResult()
	{
		return $this->result;
	}
	
	public function query()
	{
		$argv = func_get_args();
		$this->_query($argv);
		return $this->result;	
	}
	
	protected function execMethod($methodName, array $argv)
	{
		if (sizeof($argv)) {
			//  
			if ($argv[0] instanceof mysqli_result) {
				return $this->$methodName($argv[0]);
			//  ,  
			} elseif (is_string($argv[0])) {
				$this->_query($argv);
			}
		}
		//   ,         
		return $this->$methodName($this->result);
	}
	
	public function version()
	{
		return $this->li->server_info;
	}
	
	public function fetch()
	{
		$argv = func_get_args();
		return $this->execMethod('_fetch', $argv);
	}
	
	public function fetchAll()
	{
		$argv = func_get_args();
		return $this->execMethod('_fetchAll', $argv);
	}
	
	public function getColumn()
	{
		$argv = func_get_args();
		return $this->execMethod('_getColumn', $argv);
	}
	
	public function numRows()
	{
		$argv = func_get_args();
		return $this->execMethod('_numRows', $argv);
	}
	
	public function getCell()
	{
		$resType = $this->setResultType(MYSQLI_NUM);
		$argv = func_get_args();
		
		if (is_object($argv[0])) {
			$r = $this->_fetch($argv[0]);
		} elseif (is_string($argv[0])) {
			$this->_query($argv);
			$r = $this->_fetch($this->result);
		}
		$this->setResultType($resType);
		return $r[0];
	}
	
	public function affRows()
	{
		return $this->li->affected_rows;
	}
	
	public function insert()
	{
		$argv = func_get_args();
		$this->_query($argv);
		return $this->li->insert_id;
	}

	public function getInsertId()
	{
		return $this->li->insert_id;
	}
	
	public function multiQuery()
	{
		$argv = func_get_args();
		$this->_query($argv, true);
		return $this->result;
	}
	// BC
	public function formatQuery()
	{
		$argv = func_get_args();
		return $this->_prepare($argv);		
	}
	
	public function prepare()
	{
		$argv = func_get_args();
		return $this->_prepare($argv);		
	}
	
	public function reset()
	{
		if (!func_num_args())
			return $this->result->data_seek(0);
		
		$argv = func_get_args();
		if (is_object($argv[0]))
			return $argv[0]->data_seek(0);
	}
	
	public function freeResults()
	{
		do {
			if ($result = $this->li->store_result()) {
				$result->free();
			}
		} while ($this->li->next_result());
	}
	
	public function getDebugInformation()
	{
		return $this->debug ? $this->debugInformation : array();
	}
}
