<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Database_Placeholder
{
	// object of MySQLi for calling escaping
	protected $mysqli;
	// tables prefix
	protected $prefix = '';
	// data for the substitution
	protected $data = array();
	
	public function __construct(mysqli $mysqli, $prefix)
	{
		$this->mysqli = $mysqli;
		$this->prefix = $prefix;
	}
	
	protected function setPrefixes($query)
	{
		return preg_replace('#([^\\\\]):(.+?)(\W|$)#', '\\1`' . $this->prefix . '\\2`\\3', $query);
	}
	
	/**
	 * BL_Database_Placeholder::prepareQuery()
	 *    .  ,   
	 * 	 ,  
	 * 
	 * @param array $argv  (pattern[, data1[, data2[, ...]]])
	 * @return string    
	 */
	public function prepareQuery($argv)
	{
		$n = count($argv);
		if ($n == 0) {
			throw new BL_Database_Placeholder_Exception('Empty query');
		 } elseif (1 == $n && false === strpos($argv[0], '?')) {
			return $this->setPrefixes($argv[0]);
		}
		
		$query = array_shift($argv);
		$this->data = $argv;
		$query = $this->setPrefixes($query);
		
		$pattern = '#\?(\?|c@|@c|d@|@d|t@|@t|k@|@k|i@|@i|f@|@f|s@|@s|c|d|t|k|i|f|s|@)#';
		$query = preg_replace_callback($pattern, array($this, '_replacePlaceholders'), $query);
		
		if (count($this->data))
			throw new BL_Database_Placeholder_Exception('    ');
		
		return $query;
	}
	
	protected function escape($string)
	{
		return $this->mysqli->real_escape_string($string);
	}
	
	protected function _replacePlaceholders($matches)
	{
		$m = $matches[1];
		if ($m == '?') //   
			return '?';
		
		if (!count($this->data))
			throw new BL_Database_Placeholder_Exception('      ');
		//TODO: NULL
		$el = array_shift($this->data);
		switch ($m) {
			case '@c':
			case 'c@': //     ([, [,  ]])
				foreach ($el as &$e) {
					if (is_array($e)) {
						if (isset($e[2]))
							$e = '`' . $this->prefix . $e[0] . '`.`' . $e[1] . '` AS ' . $e[2];
						else
							$e = '`' . $this->prefix . $e[0] . '`.`' . $e[1] . '`' ;
					} else {
						$e = '`' . $e . '`';
					}
				}
				return implode(',', $el);
			
			case '@t':
			case 't@': //       ([,  ])
				foreach ($el as &$e) {
					if (is_array($e))
						$e = '`' . $this->prefix . $e[0] . '` AS ' . $e[1];
					else
						$e = '`' . $this->prefix . $e . '`';
				}
				return implode(',', $el);
			
			case '@k':
			case 'k@': //     
				foreach ($el as &$e)
					$e = $this->escape($e);
				return implode(', ', $el);
			
			case '@i':
			case 'i@': //  int
				foreach ($el as &$e)
					$e = ceil($e);
				return implode(', ', $el);
			
			case '@d':
			case 'd@':  //    
				 return implode(', ', $el);
			
			case '@f':
			case 'f@': //   float
				foreach ($el as &$e)
					$e = "'" . (float)$e . "'";
				return implode(', ', $el);
			
			case '@s':
			case 's@': //     
				foreach ($el as &$e)
					$e = "'" . $this->escape($e) . "'";
				return implode(', ', $el);
			
			case 'c':  //    ([, [,   ]]),   
				if (is_array($el)) {
					if (isset($el[2]))
						return '`' . $this->prefix . $el[0] . '`.`' . $el[1] . '` AS ' . $el[2];
					return '`' . $this->prefix . $el[0] . '`.`' . $el[1] . '`';
				}
				return '`' . $el . '`';
			
			case 't':  // .      
				if (is_array($el))
					return '`' . $this->prefix . $el[0] . '` AS ' . $el[1];
				return '`' . $this->prefix . $el . '`';
			
			case 'k':  //   
				 return $this->escape($el);
			
			case 'd':  //   
				 return $el;
			
			case 'i':  // int
				return ceil($el);
			
			case 'f':  //  float
				return "'" . (float)$el . "'";
			
			case 's':  // .   
				return "'" . $this->escape($el) . "'";
			
			case '@':  //   ( => ).    ,    
				$s = array();
				foreach ($el as $k => $e) {
					$s[] = '`' . $k . "`='" . $this->escape($e) . "'";
				}
				return implode(',', $s);
		}
	}
}
