<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Database_TransactionStorage
{
	/**
	 *
	 * @var BL_Database
	 */
	protected $db;

	protected $actions = array();

	public function  __construct(BL_Database $db)
	{
		$this->db = $db;
	}

	/**
	 *
	 * @param string $name
	 * @param array $arguments
	 * @return BL_Database_TransactionStorage
	 */
	public function  __call($name,  $arguments)
	{
		$this->actions[] = array($name, $arguments);
		return $this;
	}

	/**
	 *
	 * @return BL_Database_TransactionStorage
	 */
	public function query()
	{
		$this->actions[] = array('query', func_get_args());
		return $this;
	}

	public function commit()
	{
		if (!$this->actions)
			return false;

		$li = $this->db->getConnection();
		$li->autocommit(false);

		foreach ($this->actions as $action) {
			call_user_func_array(array($this->db, $action[0]), $action[1]);
		}
		$this->actions = array();

		$res = $li->commit();
		$li->autocommit(true);
		return $res;
	}

	public function rollback()
	{
		$this->actions = array();
	}

	public function  __destruct()
	{
		$this->actions && $this->commit();
	}
}
