<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Page
{
	private static $instance;
	
	protected $dirs = array(
		'javascripts' => 'js',
		'stylesheets' => 'css',
	);
	
	protected $res = array(
		'javascripts' => array(),
		'stylesheets' => array(),
	);
	
	protected $template = 'main';

	protected $data = array();
	
	protected $title = '';
	
	protected function __construct() {}
	protected function __clone() {}

	public function directories(array $dirs = array())
	{
		if ($dirs) {
			$this->dirs = array_merge($this->dirs, $dirs);
			return $this;
		}
		return $this->dirs;
	}

	/**
	 *
	 * @return BL_Page
	 */
	public static function instance()
	{
		return self::$instance ? self::$instance : self::$instance = new self();
	}
	
	public function setDirectories($dirs)
	{
		$this->dirs = array_merge($this->dirs, $dirs);
	}
	//TODO:  
	public function template($template = NULL)
	{
		if (!$template)
			return BL_Application::isBackend() ? $this->template : 'layouts/body/' . $this->template;
		
		$this->template = $template;
		return $this;
	}
	/**
	 *
	 * @param string $type
	 * @param mixed $resource
	 * @param string $revision
	 * @return BL_Page
	 */
	protected function addResource($type, $resource, $revision = NULL)
	{
		if (!($type == 'javascripts' || $type == 'stylesheets'))
			throw new BL_Exception('Unknown resource type');

		if (is_array($resource)) {
			foreach ($resource as $file => $revision) {
				if (is_integer($file)) {
					$file = $revision;
					$revision = NULL;
				}
				$this->res[$type][$file] = $revision;
			}
		} else {
			$this->res[$type][$resource] = $revision;
		}
		
		return $this;
	}
	//TODO: addJsLibrary -   jQuery/Dojo,     /    
	/**
	 *
	 * @param <type> $file
	 * @param <type> $revision
	 * @return BL_Page
	 */
	public function addJavascript($file, $revision = NULL)
	{
		return $this->addResource('javascripts', $file, $revision);
	}
	//TODO: type (media, print), multiple types
	/**
	 *
	 * @param <type> $file
	 * @param <type> $revision
	 * @return BL_Page
	 */
	public function addStylesheet($file, $revision = NULL)
	{
		return $this->addResource('stylesheets', $file, $revision);
	}
	
	protected function attachRevision($resource)
	{
		foreach ($resource as $file => $revision) {
			if (!$revision)
				$resource[$file] = BL_APP_REVISION;
		}
		
		return $resource;
	}
	
	public function getJavascripts()
	{
		return $this->attachRevision($this->res['javascripts']);
	}
	
	public function getStylesheets()
	{
		return $this->attachRevision($this->res['stylesheets']);
	}
	
	public function getJavascriptsAsString()
	{
		$str = '';
		foreach ($this->getJavascripts() as $file => $r) {
			$str .= '<script src="' . $this->dirs['javascripts'] . '/' . htmlspecialchars($file) . '.js?' . $r . '" type="text/javascript"></script>' . "\n";
		}
		return $str;
	}
	//TODO:    ( http)
	public function getStylesheetsAsString()
	{
		$str = '';
		foreach ($this->getStylesheets() as $file => $r) {
			$str .= '<link rel="stylesheet" type="text/css" href="' . $this->dirs['stylesheets'] . '/' . htmlspecialchars($file) . '.css?' . $r . '" />' . "\n";
		}
		return $str;
	}
	
	//TODO: addSmth (link for RSS)
	/**
	 *
	 * @param string $label
	 * @param mixed $data
	 * @return BL_Page
	 */
	public function setData($label, $data)
	{
		//TODO: cut 'system', 'page'
		$this->data[$label] = $data;
		return $this;
	}
	/**
	 *
	 * @param array $data
	 * @return BL_Page
	 */
	public function setDataArray(array $data)
	{
		//TODO: cut 'system', 'page'
		$this->data = array_merge($this->data, $data);
		return $this;
	}
	
	public function getData($label = NULL)
	{
		if ($label)
			return isset($this->data[$label]) ? $this->data[$label] : NULL;
		
		return $this->data;
	}
	/**
	 *
	 * @param mixed $title
	 * @return string|BL_Page
	 */
	public function title($title = NULL)
	{
		if (NULL === $title)
			return $this->title;
		
		$this->title = $title;
		return $this;
	}
	//TODO:  prepare*
	protected function prepareFrontendPage()
	{
		$data = array(
			'url'     => SITE_URL,
			'charset' => BL_APP_CHARSET
		);
		
		// 1st rendering, with calling widgets
		$page = template($this->template())
			->assign('system', $data)
			->deploy($this->data)
			->render();

		//TODO:    Page_Title
		if ($breadcrumbs = $this->getData('breadcrumbs')) {
			$breadcrumbs = array_reverse($breadcrumbs);
			foreach ($breadcrumbs as &$part) {
				$part = is_array($part) ? $part['title'] : $part;
			}
			$this->title(implode(' / ', $breadcrumbs));
		}

		$data = array(
			'system' => array(
				'url'      => SITE_URL,
				'charset'  => BL_APP_CHARSET,
				'revision' => BL_APP_REVISION
			),
			'page' => array(
				'title' => $this->title(),
				'body'  => $page,
				'js'  => $this->getJavascriptsAsString(),
				'css' => $this->getStylesheetsAsString()
			)
		);

		// second rendering
		$page = template('layouts/page/default')
			->deploy($data)
			->render();

//		if (BL_DEBUG_ENABLED)
//			$page = Debug::assignData($page);
		return $page;
	}

	protected function prepareBackendPage()
	{
		$data = array(
			'system' => array(
				'url'      => SITE_URL,
				'charset'  => BL_APP_CHARSET,
				'revision' => BL_APP_REVISION
			),
			'page' => array(
				'title' => $this->title(),
				'js'  => $this->getJavascriptsAsString(),
				'css' => $this->getStylesheetsAsString()
			)
		);

		$page = template($this->template())
				->deploy($data)
				->deploy($this->data)
				->render();
//		if (BL_DEBUG_ENABLED)
//			$page = Debug::assignData($page);
		return $page;
	}

	public function getPage()
	{
		if (class_exists('BL_Application', false)) {
			$app = BL_Application::instance();
			$app->getEventDispatcher()->notify($app->event($this, 'system.page.getPage'));
		}

		return BL_Application::isBackend() ? $this->prepareBackendPage() : $this->prepareFrontendPage();
	}
	
	public function __toString()
	{
		return $this->getPage();
	}
}
