<?php
//TODO:    (eastic)
//TODO: jumping style http://pear.php.net/manual/en/package.html.pager.compare.php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

/**
 * Paginator class
 *
 * @package Bicycle Libraries
 * @author  Konstantin.Myakshin
 * @link    http://brouzie.com/projects/bicycle-libraries/user-guide/paginator.html
 */
class BL_Paginator
{
	//  
	protected $count = 0;
	//  
	protected $countPages = 0;
	//  
	protected $currentPage = 1;
	
	const FIRST = 1;
	
	protected $settings = array(
		'perPage'   => 25,
		'limited'   => true,
		// 2, 3 .. 5, 6 -> 2, 3, 4, 5, 6
		'linkNear'  => true,
		//         2  
		'hideFL'    => false,
		'maxLeft'   => 5,
		'maxMiddle' => 5,
		'maxRight'  => 5
	);
	
	public function __construct($count, $currentPage = 1, array $settings = array())
	{
		$this->count = ceil($count);
		$this->currentPage = $currentPage;
		$this->setSettings($settings);
	}
	
	public function setSettings(array $settings)
	{
		$perPage = $this->settings['perPage'];
		$this->settings = array_merge($this->settings, $settings);
		if (!$this->settings['perPage'])
			$this->settings['perPage'] = $perPage;
		
		$this->countPages = ceil(($this->count) / $this->settings['perPage']);
		
		$this->correctPage($this->currentPage);
	}
	
	protected function correctPage(&$page)
	{
		$page = ceil($page);
		if ($page > $this->countPages || $page == -1)
			$page = $this->countPages;
			
		if ($page < self::FIRST)
			$page = self::FIRST;
	}
	
	public function getCurrentPage($full = false)
	{
		return $full ? $this->setFromTo($this->currentPage) : $this->currentPage;
	}
	
	public function getCountPages()
	{
		return $this->countPages;
	}
	
	public function getCountElements()
	{
		return $this->count;
	}
	
	public function getCountElementsOnPage($page = NULL)
	{
		$page = $page ? $page : $this->currentPage;
		$this->correctPage($page);
		
		return ($page < $this->countPages) ?
			$this->settings['perPage'] : $this->count % ($this->countPages - 1);
	}
	//   
	public function getFirstEl($page = NULL)
	{
		$page = $page ? $page : $this->currentPage;
		return ($page - 1) * $this->settings['perPage'];
	}
	
	public function getPerPage()
	{
		return $this->settings['perPage'];
	}
	
	public function getFirstPage($full = false)
	{
		if ($this->settings['hideFL'] && (self::FIRST == $this->currentPage - 1))
			return false;
		
		if (self::FIRST == $this->currentPage)
			return false;
		
		return $full ? $this->setFromTo(self::FIRST) : self::FIRST;
	}
	
	public function getPrevPage($full = false)
	{
		if ($this->currentPage == self::FIRST)
			return false;
		
		$page = ($this->currentPage > self::FIRST) ? $this->currentPage - 1 : 1;
		return $full ? $this->setFromTo($page) : $page;
	}
	
	public function getNextPage($full = false)
	{
		if ($this->currentPage == $this->countPages)
			return false;
		
		$page = ($this->currentPage + 1 < $this->countPages) ? $this->currentPage + 1 : $this->countPages;
		return $full ? $this->setFromTo($page) : $page;
	}
	
	public function getLastPage($full = false)
	{
		if ($this->settings['hideFL'] && ($this->countPages == $this->currentPage + 1))
			return false;
		
		if ($this->countPages == $this->currentPage)
			return false;
		
		return $full ? $this->setFromTo($this->countPages) : $this->countPages;
	}
	
	protected function getLeftPages()
	{
		$pages = array();
		$i = self::FIRST;
		while ($i <= $this->settings['maxLeft'] && $i < $this->countPages) {
			array_push($pages, $i);
			$i++;
		}
		return $pages;
	}
	
	protected function getMiddlePages()
	{
		$pages = array();
		if ($this->currentPage + 1 < $this->settings['maxLeft'])
			return $pages;
		
		$i = $this->currentPage;
		array_push($pages, $i);
		$c = 1;
		$d = 1;
		$left = true;
		while ($c < $this->settings['maxMiddle']) {
			if ($left) {
				$i = $this->currentPage - $d;
				$left = false;
			} else {
				$i = $this->currentPage + $d;
				$left = true;
				$d++;
				if ($i > $this->countPages)
					continue;
			}
			array_push($pages, $i);
			$c++;
		}
		sort($pages);
		return $pages;
	}
	
	protected function getRightPages()
	{
		$pages = array();
		$i = $this->countPages;
		$c = 0;
		while ($c < $this->settings['maxRight']) {
			array_push($pages, $i);
			$i--;
			$c++;
		}
		return array_reverse($pages);
	}
	//   
	protected function setFromTo($page)
	{
		$from = $this->getFirstEl($page) + 1;
		$to   = $from + $this->getCountElementsOnPage($page) - 1;
		return array($page => array($from, $to));
	}
	
	public function getPages($full = false)
	{
		$allLimits = $this->settings['maxLeft'] + $this->settings['maxMiddle']
			+ $this->settings['maxRight'];
		
		$pages = array(
			'left'   => array(),
			'middle' => array(),
			'right'  => array()
		);
		
		if ((!$this->settings['limited'] || $this->countPages <= $allLimits) && $this->countPages) {
			$pages['left'] = range(self::FIRST, $this->countPages);
			
			if (!$full)
				return $pages;
			
			foreach ($pages['left'] as &$page) {
				$page = $this->setFromTo($page);
			}
			return $pages;
		}
		
		$left   = $this->getLeftPages();
		$middle = $this->getMiddlePages();
		$right  = $this->getRightPages();
		
		$minMiddle = (empty($middle)) ? 0 : min($middle);
		$minRight  = (empty($right))  ? 0 : min($right);
		
		if (anyInArray(array($minMiddle, $minMiddle - 1), $left)) {
			if (anyInArray(array($minRight, $minRight - 1), $middle)) {
				$middle = array_unique(array_merge($middle, $right));
				$right = array();
			}
			$left = array_unique(array_merge($left, $middle));
			$middle = array();
		} elseif(anyInArray(array($minRight, $minRight - 1), $middle)) {
			$middle = array_unique(array_merge($middle, $right));
			$right = array();
		}
		
		if ($this->settings['linkNear']) {
			if (in_array($minMiddle - 2, $left)) {
				if (in_array($minRight - 2, $middle)) {
					array_push($middle, $minRight - 1);
					$middle = array_unique(array_merge($middle, $right));
					$right = array();
				}
				array_push($left, $minMiddle - 1);
				$left = array_unique(array_merge($left, $middle));
				$middle = array();
			} elseif (in_array($minRight - 2, $middle)) {
				array_push($middle, $minRight - 1);
				$middle = array_unique(array_merge($middle, $right));
				$right = array();
			}
		}
		
		$pages = array(
			'left'   => $left,
			'middle' => $middle,
			'right'  => $right
		);
		
		if (!$full)
			return $pages;
		
		foreach ($pages['left'] as &$page) {
			$page = $this->setFromTo($page);
		}
		
		foreach ($pages['middle'] as &$page) {
			$page = $this->setFromTo($page);
		}
		
		foreach ($pages['right'] as &$page) {
			$page = $this->setFromTo($page);
		}
		
		return $pages;
	}
}
