<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

//TODO:  
//TODO:  URL      
//TODO:  
//TODO: , 
class BL_Router
{
	protected $url = '';
	/**
	 *
	 * @var BL_Request
	 */
	protected $request;
	
	protected static $parsed = false;
	
	protected static $isFirst;
	
	protected static $is404;
	
	protected function __construct(BL_Request $request, $url = NULL)
	{
		$this->request = $request;
		$this->url = $url ? $url : $this->getDefaultUrl();
	}
	
	protected function getDefaultUrl()
	{
		$uri = explode('/', $this->request->getRequestUri());
		$sn  = explode('/', $_SERVER['SCRIPT_NAME']);
		$offset = 0;
		
		foreach ($sn as $i => $part) {
			if (!isset($uri[$i]) || $uri[$i] != $part) {
				break;
			}
			$offset++;
		}
		
		return implode('/', array_slice($uri, $offset));
	}
	
	protected function _parse(array $parsers)
	{
		$e404 = true;
		foreach ($parsers as &$parser) {
			if (is_string($parser))
				$parser = new $parser();
			
			if (!$parser instanceof BL_Router_Interface)
				throw new BL_Exception(sprintf('"%s" must implement BL_Router_Interface', get_class($parser)));
			
			$data = call_user_func_array(array($parser, 'parse'), array(&$this->url));
			if (is_array($data)) {
				$this->request->deploy($data);
				$e404 = false;
				// we have matching router, stop parsing
				break;
			}
		}
		
		if (!isset(self::$is404)) {
			self::$is404 = $e404;
		}
		
		if (!(isset(self::$isFirst) && self::$parsed)) {
			self::$isFirst = !$this->request->request('m', self::$is404);
		}
	}
	
	public static function is404()
	{
		return self::$is404;
	}
	
	public static function isFirstPage()
	{
		return self::$isFirst;
	}

	public function getUri()
	{
		return $this->url;
	}
	//TODO:      
	public static function parse(BL_Request $request, $url = NULL, $parsers = array())
	{
		if (self::$parsed && !($parsers && $url)) {
			throw new BL_Exception('Main URL already has parsed');
		}
		
		if (!$parsers) {
			$parsers = array(
				'BL_Router_Common',
				'BL_Router_SemiDynamic',
				'BL_Router_Database'
			);
		}
		
		$instance = new self($request, $url);
		$instance->_parse((array)$parsers);
		self::$parsed = true;
	}
}
