<?php

class BL_Session_Namespace implements ArrayAccess
{
	const FLASH = '__flash__';

	/**
	 *
	 * @var BL_Session
	 */
	protected $master;

	/**
	 *
	 * @var string
	 */
	protected $name;

	/**
	 *
	 * @var array
	 */
	protected $data;

	/**
	 *
	 * @param BL_Session $sessionObj
	 * @param string $name
	 * @param $data
	 */
	public function __construct(BL_Session_Light $sessionObj, $name, &$data)
	{
		empty($data[$name]) && $data[$name] = array();

		$this->master = $sessionObj;
		$this->name   = $name;
		$this->data   =& $data[$name];

		isset($this->data[self::FLASH]) || $this->data[self::FLASH] = array();
	}

	/**
	 *
	 * @return string
	 */
	public function getName()
	{
		return $this->name;
	}

	public function set($key, $value)
	{
		$this->data[$key] = $value;
		return $this;
	}

	public function get($key, $default)
	{
		return isset($this->data[$key]) ? $this->data[$key] : $default;
	}

	//TODO: __get/__set
	
	public function offsetGet($offset)
	{
		return isset($this->data[$offset]) ? $this->data[$offset] : NULL;
	}

	public function offsetExists($offset)
	{
		return array_key_exists($offset, $this->data);
	}
	
	public function offsetSet($offset, $value)
	{
		$this->data[$offset] = $value;
	}

	public function offsetUnset($offset)
	{
		unset($this->data[$offset]);
	}

	/**
	 *
	 * @param string $name
	 * @return BL_Session_Namespace
	 */
	public function ns($name)
	{
		return $this->master->ns($name);
	}

	/**
	 *
	 * @param string|array $message
	 * @param string $token
	 * @return BL_Session_Namespace
	 */
	public function setFlash($message, $token = 'default')
	{
		$this->data[self::FLASH][$token] = $message;
		return $this;
	}

	public function flash($message, $token = 'default', $remove = true)
	{
		if (isset($this->data[self::FLASH][$token])) {
			$return = $this->data[self::FLASH][$token];
			if ($remove) {
				unset($this->data[self::FLASH][$token]);
			}
			return $return;
		}
		return NULL;
	}

	/**
	 *
	 * @return array
	 */
	public function getData()
	{
		return $this->data;
	}

	/**
	 *
	 * @return BL_Session
	 */
	public function master()
	{
		return $this->master;
	}

	/**
	 *
	 * @return BL_Session_Namespace
	 */
	public function flush()
	{
		$this->data = array(self::FLASH => array());
		return $this;
	}
}
