<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Template
{
	const DATA     = 1;
	const TEMPLATE = 2;
	const ALL      = 3;
	
	const DEFAULT_EXT = 'htm';
	
	protected $template = NULL;
	
	protected $data = array();
	
	protected $templateData;
	
	protected $baseDir = '';
	
	protected static $envs = array();
	/**
	 *
	 * @param string $name
	 * @return Twig_Environment
	 */
	protected static function createEnv($name)
	{
		$_settings = BL_Settings::get('template');
		$options = array(
			'base_template_class' => 'BL_Template_BaseTemplateClass',
			'debug'       => BL_DEBUG_ENABLED,
			'charset'     => BL_APP_CHARSET,
			'trim_blocks' => true,
			'auto_reload' => $_settings['cache']['reload'],
			'cache'       => $_settings['dirs']['cache']
		);
		
		if ($name == 'string') {
			$loader = new Twig_Loader_String();
		} else {
			$loader = new Twig_Loader_Filesystem($_settings['dirs']['templates']);
		}
		
		$twig = new Twig_Environment($loader, $options);
		
		$tags = $_settings['tags'];
		$options = array(
			'tag_comment'  => array($tags['comment']['open'], $tags['comment']['close']),
			'tag_block'    => array($tags['cs']['open'], $tags['cs']['close']),
			'tag_variable' => array($tags['var']['open'], $tags['var']['close'])
		);
		$twig->setLexer(new Twig_Lexer($twig, $options));
		
		$twig->setExtensions(array(new BL_Template_Extension_Core(), new Twig_Extension_Escaper()));
		
		return $twig;
	}

	/**
	 *
	 * @param string $name
	 * @return BL_Template_Environment
	 */
	protected static function env($name)
	{
		return isset(self::$envs[$name]) ? self::$envs[$name] : self::$envs[$name] = self::createEnv($name);
	}
	/**
	 *
	 * @param string $dir
	 * @return BL_Template|string
	 */
	public function baseDir($dir = NULL)
	{
		if (NULL === $dir)
			return $this->baseDir;
		
		$this->baseDir = $dir ? $dir . '/' : '';
		return $this;
	}
	/**
	 *
	 * @param string $template
	 * @param string $ext
	 * @return BL_Template
	 */
	public function load($template, $ext = self::DEFAULT_EXT)
	{
		$this->template = $template . '.' . $ext;
		
		return $this;
	}
	/**
	 *
	 * @param string $data
	 * @return BL_Template
	 */
	public function loadData($data)
	{
		$this->templateData = $data;
		return $this;
	}
	/**
	 *
	 * @param mixed $data
	 * @param string $prefix
	 * @return BL_Template
	 */
	public function deploy($data, $prefix = NULL)
	{
		if ($prefix) {
			foreach ((array)$data as $key => $value) {
				$this->data[$prefix . $key] = $value;
			}
		} else {
			$this->data = array_merge($this->data, $data);
		}
		
		return $this;
	}
	//TODO: block flag
	//TODO: nested labels
	//TODO: global var
	/**
	 *
	 * @param string $label
	 * @param mixed $data
	 * @return BL_Template
	 */
	public function assign($label, $data)
	{
		if (strpos($label, '.') === false) {
			$this->data[$label] = $data;
		} else {
			$_data = $data;
			$labels = explode('.', $label);
			for ($i = sizeof($labels); $i >= 0; $i--) {
				if (isset($labels[$i]))
					$_data = array($labels[$i] => $_data);
			}
			//TODO: fix it
			$this->data = array_merge_recursive_distinct($this->data, $_data);
		}
		return $this;
	}
	
	/**
	 * alias of "assign"
	 *
	 * @param string $label
	 * @param mixed $data
	 * @return BL_Template
	 */
	public function v($label, $data)
	{
		return $this->assign($label, $data);
	}
	
	public function getData($label)
	{
		return isset($this->data[$label]) ? $this->data[$label] : NULL;
	}
	/**
	 *
	 * @param int $type
	 * @return BL_Template
	 */
	public function clear($type = self::DATA)
	{
		if ($type == self::DATA || $type == self::ALL)
			$this->data = array();
		
		if ($type == self::TEMPLATE || $type == self::ALL)
			$this->template = NULL;
		
		return $this;
	}
	public function render($template = NULL, $ext = self::DEFAULT_EXT)
	{
		if (isset($this->templateData)) {
			$env = 'string';
			$template = $this->templateData;
		} else {
			$template = $template ? $template . '.' . $ext : $this->template;
			$template = $this->baseDir . $template;
			$env = 'file';
		}

		return self::env($env)
			->loadTemplate($template)
			->render($this->data);
	}

	public function __toString()
	{
		return $this->render();
	}
}
