<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */
//TODO: smart fallbacks (if translation not exists in lang1 use lang2 or call method + notify event or try use another driver)
//TODO: locale -> langCode
class BL_Translate
{
	protected static $instances = array();

	/**
	 *
	 * @var BL_Translate_DriverAbstract
	 */
	protected $driver;

	protected $langCode = 'en';

	protected $namespace;

	protected static $settings = array();
	
	protected function __construct($namespace, BL_Translate_DriverAbstract $driver)
	{
		$this->namespace = $namespace;
		$this->driver    = $driver;

		if (!is_callable(self::$settings['fallback']))
			throw new BL_Exception('Fallback is not valid callback');
	}

	public static function isDriverAvalible($driver)
	{
		$exists = false;
		try {
			$exists = (class_exists($driver, true) || class_exists($driver = 'BL_Translate_Driver_' . ucfirst($driver), true))
					&& in_array('BL_Translate_DriverAbstract', class_parents($driver)) ? $driver : false;
		} catch (BL_Loader_Exception $e) {
			// do nothing
		}
		return $exists;
	}

	public static function setDefaultSettings(array $settings)
	{
		self::$settings = array_merge_recursive_distinct(self::$settings, $settings);
	}

	/**
	 *
	 * @param string $namespace
	 * @param BL_Translate_DriverAbstract|string|NULL $driver
	 * @param array $settings driver settings
	 * @return BL_Translate
	 */
	public static function instance($namespace, $driver = NULL, array $settings = array())
	{
		//TODO: remove
		if (!self::$settings) {
			self::$settings = BL_Settings::getMerged(array(
				'fallback'       => array('BL_Translate', 'fallback'),
				'defaultDriver'  => 'BL_Translate_Driver_Array'
			), 'translate');
		}
		//TODO:  :     $instances    
		if (NULL === $driver) {
			$driver = new self::$settings['defaultDriver']($settings);
		} elseif (is_string($driver) && $driver = self::isDriverAvalible($driver)) {
			$driver = new $driver($settings);
		}
		
		return isset(self::$instances[$namespace]) ? self::$instances[$namespace] : self::$instances[$namespace] = new self($namespace, $driver);
	}
	//TODO: setLocale (object)
	public function setLanguage($code)
	{
		$this->langCode = $code;
		return $this;
	}

	public function isTranslated($string, $count = NULL, $namespace = NULL, $locale = NULL)
	{
		$string = mb_strtolower($string, BL_APP_CHARSET);
		$namespace || $namespace = $this->namespace;
		$locale    || $locale    = $this->langCode;
		$pluralForm = NULL === $count ? -1 : self::getPluralForm($count, $locale);
		
		return $this->driver->isTranslated($string, $pluralForm, $namespace, $locale);
	}

	protected function translate($string, $count, $args = array(), $namespace = NULL, $locale = NULL)
	{
		$string = mb_strtolower($string, BL_APP_CHARSET);
		$namespace || $namespace = $this->namespace;
		$locale    || $locale    = $this->langCode;
		$pluralForm = NULL === $count ? -1 : self::getPluralForm($count, $locale);
		
		if (!$text = $this->driver->getTranslation($string, $pluralForm, $namespace, $locale))
			$text = call_user_func_array(self::$settings['fallback'], array($string, $pluralForm, $namespace, $locale));
		
		$args && $text = vsprintf($text, $args);
		return $text;
	}

	protected static function fallback($string, $pluralForm, $namespace, $locale)
	{
		return sprintf('*%s|%s*', $namespace, $string);
	}

	public function tr($string, $args = array(), $namespace = NULL, $locale = NULL)
	{
		return $this->translate($string, NULL, $args, $namespace, $locale);
	}
	
	public function ntr($string, $count, $args = array(), $namespace = NULL, $locale = NULL)
	{
		return $this->translate($string, $count, $args, $namespace, $locale);
	}

	public function getUntranslatedTokens($namespace = NULL)
	{
		$namespace || $namespace = $this->namespace;
		$tokens = $this->driver->getTokens($namespace);

		$return = array();
		foreach ($tokens as $token => $isPlural) {
			//TODO:     (  self::getPluralFormsCount())
			if (!$this->isTranslated($token, $isPlural ? 0 : -1)) {
				$return[] = $token;
			}
		}
		return $return;
	}

	/**
	 *
	 * @return BL_Translate_DriverAbstract|BL_Translate_DriverWriterInterface
	 */
	public function getDriver()
	{
		return $this->driver;
	}

	/**
	 * @author Zend Framework team
	 * @param int $n
	 * @param string $langCode
	 * @return int
	 */
	public static function getPluralForm($n, $langCode)
	{
		switch ($langCode) {
			case 'bo':
			case 'dz':
			case 'id':
			case 'ja':
			case 'jv':
			case 'ka':
			case 'km':
			case 'kn':
			case 'ko':
			case 'ms':
			case 'th':
			case 'tr':
			case 'vi':
			case 'zh':
				return 0;
				break;

			case 'af':
			case 'az':
			case 'bn':
			case 'bg':
			case 'ca':
			case 'da':
			case 'de':
			case 'el':
			case 'en':
			case 'eo':
			case 'es':
			case 'et':
			case 'eu':
			case 'fa':
			case 'fi':
			case 'fo':
			case 'fy':
			case 'gl':
			case 'gu':
			case 'ha':
			case 'he':
			case 'hu':
			case 'is':
			case 'it':
			case 'ku':
			case 'lb':
			case 'ml':
			case 'mn':
			case 'mr':
			case 'nb':
			case 'ne':
			case 'nl':
			case 'nn':
			case 'no':
			case 'om':
			case 'or':
			case 'pa':
			case 'ps':
			case 'pt':
			case 'so':
			case 'sq':
			case 'sv':
			case 'sw':
			case 'ta':
			case 'te':
			case 'tk':
			case 'ur':
			case 'zu':
				return ($n == 1) ? 0 : 1;

			case 'am':
			case 'bh':
			case 'fr':
			case 'hi':
			case 'ln':
			case 'mg':
			case 'ti':
			case 'wa':
				return (($n == 0) || ($n == 1)) ? 0 : 1;

			case 'be':
			case 'bs':
			case 'hr':
			case 'ru':
			case 'sr':
			case 'uk':
				return (($n % 10 == 1) && ($n % 100 != 11)) ? 0 : ((($n % 10 >= 2) && ($n % 10 <= 4) && (($n % 100 < 10) || ($n % 100 >= 20))) ? 1 : 2);

			case 'cs':
			case 'sk':
				return ($n == 1) ? 0 : ((($n >= 2) && ($n <= 4)) ? 1 : 2);

			case 'ga':
				return ($n == 1) ? 0 : (($n == 2) ? 1 : 2);

			case 'lt':
				return (($n % 10 == 1) && ($n % 100 != 11)) ? 0 : ((($n % 10 >= 2) && (($n % 100 < 10) || ($n % 100 >= 20))) ? 1 : 2);

			case 'sl':
				return ($n % 100 == 1) ? 0 : (($n % 100 == 2) ? 1 : ((($n % 100 == 3) || ($n % 100 == 4)) ? 2 : 3));

			case 'mk':
				return ($n % 10 == 1) ? 0 : 1;

			case 'mt':
				return ($n == 1) ? 0 : ((($n == 0) || (($n % 100 > 1) && ($n % 100 < 11))) ? 1 : ((($n % 100 > 10) && ($n % 100 < 20)) ? 2 : 3));

			case 'lv':
				return ($n == 0) ? 0 : ((($n % 10 == 1) && ($n % 100 != 11)) ? 1 : 2);

			case 'pl':
				return ($n == 1) ? 0 : ((($n % 10 >= 2) && ($n % 10 <= 4) && (($n % 100 < 10) || ($n % 100 > 29))) ? 1 : 2);

			case 'cy':
				return ($n == 1) ? 0 : (($n == 2) ? 1 : ((($n == 8) || ($n == 11)) ? 2 : 3));

			case 'ro':
				return ($n == 1) ? 0 : ((($n == 0) || (($n % 100 > 0) && ($n % 100 < 20))) ? 1 : 2);

			case 'ar':
				return ($n == 0) ? 0 : (($n == 1) ? 1 : (($n == 2) ? 2 : ((($n >= 3) && ($n <= 10)) ? 3 : ((($n >= 11) && ($n <= 99)) ? 4 : 5))));

			default:
				return 0;
		}
	}
	/**
	 * @param string $langCode
	 * @return int
	 */
	public static function getPluralFormsCount($langCode)
	{
		switch ($langCode) {
			case 'bo':
			case 'dz':
			case 'id':
			case 'ja':
			case 'jv':
			case 'ka':
			case 'km':
			case 'kn':
			case 'ko':
			case 'ms':
			case 'th':
			case 'tr':
			case 'vi':
			case 'zh':
				return 1;
				break;
			
			case 'af':
			case 'az':
			case 'bn':
			case 'bg':
			case 'ca':
			case 'da':
			case 'de':
			case 'el':
			case 'en':
			case 'eo':
			case 'es':
			case 'et':
			case 'eu':
			case 'fa':
			case 'fi':
			case 'fo':
			case 'fy':
			case 'gl':
			case 'gu':
			case 'ha':
			case 'he':
			case 'hu':
			case 'is':
			case 'it':
			case 'ku':
			case 'lb':
			case 'ml':
			case 'mn':
			case 'mr':
			case 'nb':
			case 'ne':
			case 'nl':
			case 'nn':
			case 'no':
			case 'om':
			case 'or':
			case 'pa':
			case 'ps':
			case 'pt':
			case 'so':
			case 'sq':
			case 'sv':
			case 'sw':
			case 'ta':
			case 'te':
			case 'tk':
			case 'ur':
			case 'zu':
			case 'am':
			case 'bh':
			case 'fr':
			case 'hi':
			case 'ln':
			case 'mg':
			case 'ti':
			case 'wa':
			case 'mk':
				return 2;
			
			case 'be':
			case 'bs':
			case 'hr':
			case 'ru':
			case 'sr':
			case 'uk':
			case 'cs':
			case 'sk':
			case 'ga':
			case 'lt':
			case 'lv':
			case 'pl':
			case 'ro':
				return 3;
			
			case 'mt':
			case 'cy':
			case 'sl':
				return 4;
			
			case 'ar':
				return 6;
			
			default:
				return 0;
		}
	}

	public static function getPluralFormExamples($langCode, $limit = 3)
	{
		$count = self::getPluralFormsCount($langCode);
		$example = array();
		$i = 0;

		while (sizeof($example) < $count) {
			$form = self::getPluralForm($i, $langCode);
			if (!isset($example[$form])) {
				$example[$form] = array($i);
			} elseif (count($example[$form]) < $limit) {
				$example[$form][] = $i;
			}
			$i++;
		}
		ksort($example);
		
		return $example;
	}
}
