<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Translate_Driver_Array extends BL_Translate_DriverAbstract implements BL_Translate_DriverWriterInterface
{
	protected $settings = array(
		'dir'       => 'translations/array',
		'serialize' => true //TODO:  
	);

	// array('namespace' => array('ru' => true))
	protected $changed = array();

	protected function getSettingsToken()
	{
		return 'translate.array';
	}

	protected function loadData($namespace, $langCode)
	{
		$this->data[$namespace][$langCode] = $this->isAvalible($namespace, $langCode) ? include $this->getFileName($namespace, $langCode) : array();
		
		foreach ($this->data[$namespace][$langCode] as &$translation) {
			is_array($translation) ||	$translation = array(-1 => $translation);
		}
	}

	public function injectTranslations($namespace, $langCode, array $translations)
	{
		$this->markChanged($namespace, $langCode);
		$this->data[$namespace][$langCode] = $translations;
	}

	public function getTokens($namespace)
	{
		if (!isset($this->tokens[$namespace])) {
			if (!$this->isAvalible($namespace, $this->masterLangCode))
				throw new BL_Exception('Missing translation for master language (' . $this->masterLangCode . ')');

			$masterTr = include $this->getFileName($namespace, $this->masterLangCode);
			$this->tokens[$namespace] = array();

			foreach ($masterTr as $tr) {
				if (is_array($tr)) {
					$this->tokens[$namespace][$tr] = !(1 == count($tr) && isset($tr[-1]));
				} else {
					$this->tokens[$namespace][$tr] = false;
				}
			}
		}

		return $this->tokens[$namespace];
	}

	protected function getFolderName($namespace)
	{
		return $this->settings['dir'] . '/' . str_replace('.', '/', $namespace) . '/';
	}

	protected function getFileName($namespace, $langCode)
	{
		return $this->getFolderName($namespace) . $langCode . '.php';
	}

	public function isAvalible($namespace, $langCode)
	{
		return file_exists($this->getFileName($namespace, $langCode));
	}

	protected function markChanged($namespace, $langCode)
	{
		isset($this->changed[$namespace]) || $this->changed[$namespace] = array();
		$this->changed[$namespace][$langCode] = true;
	}

	public function addToken($namespace, $token, $isPlural = false)
	{
		$this->tokens[$namespace][$token] = $isPlural;
	}

	public function addTranslation($namespace, $token, $langCode, $pluralForm, $translation)
	{
		$this->data[$namespace][$langCode][$token][$pluralForm] = $translation;
		$this->markChanged($namespace, $langCode);
	}

	public function removeNamespace($namespace, $langCode = NULL)
	{
		if ($langCode) {
			$file = $this->getFileName($namespace, $langCode);
			unlink($file);
		} else {
			$dir = $this->getFolderName($namespace);
			foreach (glob($dir . '*') as $file) {
				unlink($file);
			}
			rmdir($dir);
		}
	}

	public function removeToken($namespace, $token, $langCode = NULL, $pluralForm = NULL)
	{
		throw new BL_Exception('This driver doesn\'t support this feature yet');
	}

	public function save()
	{
		foreach ($this->changed as $namespace => $changed) {
			is_dir($dir = $this->getFolderName($namespace)) || mkdir($dir, 0777, true);

			foreach (array_keys($changed) as $langCode) {
				if (!$this->data[$namespace][$langCode])
					continue; // skip empty translations

				$data = "<?php\n\n" . 'return ' . var_export($this->data[$namespace][$langCode], true) . ";\n";
				file_put_contents($this->getFileName($namespace, $langCode), $data);
			}
		}

		$this->changed = array();
	}
}
