<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Translate_Driver_Database extends BL_Translate_DriverAbstract implements BL_Translate_DriverWriterInterface
{
	/**
	 *
	 * @var BL_Database
	 */
	protected $db;

	public function __construct(array $settings = array())
	{
		$this->db = BL_Context::instance()->db();
	}

	protected function loadData($namespace, $langCode)
	{
		$this->data[$namespace][$langCode] = array();

		$this->db->query('
			SELECT t.*, tt.*
			FROM :translations_tokens AS tt
				LEFT JOIN :translations t ON (t.namespace=tt.namespace AND t.token=tt.token)
			WHERE tt.namespace=?s AND t.lang_code=?s
		', $namespace, $langCode);

		while ($row = $this->db->fetch()) {
			$this->tokens[$namespace][$row['string']] = $row['is_plural'];
			$this->data[$namespace][$langCode][$row['string']][$row['plural_form']] = $row['value'];
		}
	}

	/**
	 *
	 * @return BL_Database_TransactionStorage
	 */
	protected function db()
	{
		//return $this->db;
		return $this->db->transaction(get_class($this));
	}
	
	public function addToken($namespace, $token, $isPlural = false)
	{
		$data = array(
			'namespace' => $namespace,
			'token'     => md5($token),
			'string'    => $token,
			'is_plural' => (int)$isPlural
		);

		$this->db()->query('INSERT INTO :translations_tokens SET ?@ ON DUPLICATE KEY UPDATE is_plural=?i', $data, $isPlural);
	}
	
	public function addTranslation($namespace, $token, $langCode, $pluralForm, $translation)
	{
		$data = array(
			'namespace'   => $namespace,
			'token'       => md5($token),
			'lang_code'   => $langCode,
			'plural_form' => $pluralForm,
			'value'       => $translation
		);

		$this->db()->query('INSERT INTO :translations SET ?@', $data);
	}

	public function updateTranslationRT($namespace, $tokenMd5, $langCode, $pluralForm, $translation)
	{
		$this->db()->query('UPDATE :translations SET value=?s
			WHERE namespace=?s AND token=?s AND lang_code=?s AND plural_form=?i',
			$translation, $namespace, $tokenMd5, $langCode, $pluralForm);
	}
	
	public function removeNamespace($namespace, $langCode = NULL)
	{
		//TODO: remove it from data
		if ($langCode) {
			$this->db()->query('DELETE FROM :translations WHERE namespace=?s AND lang_code=?s', $namespace, $langCode);
		} else {
			$this->db()
				->query('DELETE FROM :translations WHERE namespace=?s', $namespace)
				->query('DELETE FROM :translations_tokens WHERE namespace=?s', $namespace);
		}
	}
	
	public function removeToken($namespace, $token, $langCode = NULL, $pluralForm = NULL)
	{
		$token = md5($token);
		$this->removeTokenRT($namespace, $token, $langCode, $pluralForm);
	}

	public function removeTokenRT($namespace, $tokenMd5, $langCode = NULL, $pluralForm = NULL)
	{
		if (NULL === $pluralForm) {
			if ($langCode) {
				$this->db()->query('DELETE FROM :translations WHERE namespace=?s AND token=?s AND lang_code=?s', $namespace, $tokenMd5, $langCode);
			} else {
				$this->db()
					->query('DELETE FROM :translations WHERE namespace=?s AND token=?s', $namespace, $tokenMd5)
					->query('DELETE FROM :translations_tokens WHERE namespace=?s AND token=?s', $namespace, $$tokenMd5);
			}
		} else {
			$this->db()->query('
				DELETE FROM :translations
				WHERE namespace=?s AND token=?s AND lang_code=?s AND plural_form=?i
				', $namespace, $$tokenMd5, $langCode, $pluralForm);
		}
	}
	
	public function save()
	{
		$this->data = array(); // reset data
		$this->db()->commit();
	}
}
