<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

abstract class BL_Translate_DriverAbstract
{
	// array('namespace' => array('ru' => array('string 1' => array(-1 => 'translation 1'))))
	protected $data = array();
	// array('namespace' => array('token' => bool is plural))
	protected $tokens = array();

	protected $settings = array();
	//TODO: getter/setter for this variable
	protected $masterLangCode = 'ru';

	public function __construct(array $settings = array())
	{
		$_settings = ($token = $this->getSettingsToken()) ? BL_Settings::get($token) : array();
		$this->settings = array_merge($this->settings, $_settings, $settings);
	}

	protected function getSettingsToken()
	{
		return false;
	}

	abstract protected function loadData($namespace, $langCode);

	protected function checkLoad($namespace, $langCode)
	{
		if (!isset($this->data[$namespace][$langCode]))
			$this->loadData($namespace, $langCode);
	}

	public function isTranslated($string, $pluralForm, $namespace, $langCode)
	{
		$this->checkLoad($namespace, $langCode);
		return !empty($this->data[$namespace][$langCode][$string][$pluralForm]);
	}

	public function getTokens($namespace)
	{
		$this->checkLoad($namespace, $langCode);
		return $this->tokens[$namespace];
	}

	public function getTranslation($string, $pluralForm, $namespace, $langCode)
	{
		return $this->isTranslated($string, $pluralForm, $namespace, $langCode) ? $this->data[$namespace][$langCode][$string][$pluralForm] : false;
	}

	public function getTranslations($namespace, $langCode)
	{
		$this->checkLoad($namespace, $langCode);
		return $this->data[$namespace][$langCode];
	}

	public function canWrite()
	{
		return $this instanceof BL_Translate_DriverWriterInterface;
	}

	public function __destruct()
	{
		$this->canWrite() && $this->save();
	}
}
