<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

abstract class BL_Validator_Abstract
{
	protected $messages = array();

	/**
	 *
	 * @var BL_Validator
	 */
	protected $validator;

	/**
	 *
	 * @param string $rule
	 * @param string $field
	 * @param mixed $value
	 * @param string $error
	 * @param array $messages
	 * @param array $options
	 * @return bool is this field valid
	 */
	abstract public function validate($rule, $field, &$value, &$error, array $messages = array(), array $options = array());
	
	/**
	 * @return array array or rules wich this class can check
	 */
	abstract public function getRules();
	
	public function setValidator(BL_Validator $validator)
	{
		$this->validator = $validator;
	}

	public static function init()
	{
		//TODO: check this, use reflection for creating instances
		$className = get_call_stack();
		return new $className();
	}
	/**
	 *
	 * @param array $messages
	 * @return BL_Validator_Abstract
	 */
	public function setMessages(array $messages)
	{
		$this->messages = array_merge_recursive_distinct($this->messages, $messages);
		return $this;
	}
	
	protected function getOption(array $options, $key, $altKey = 0, array $defaultOptions = array())
	{
		foreach ($defaultOptions as $k => $v) {
			if (!isset($options[$k]))
				$options[$k] = $v;
		}

		// only by field
		if ($altKey == -1) {
			if (isset($options[$key])) {
				return $options[$key];
			}
			throw new BL_Exception('Error in options');
		}
		
		if (isset($options[$key]))
			return $options[$key];
		
		if (isset($options[$altKey]))
			return $options[$altKey];
		
		throw new BL_Exception('Error in options');
	}
	//TODO: alias of message
	protected function getMessage($rule, array $messages = array(), $errorCode = false)
	{
		if (!isset($this->messages[$rule]))
			throw new BL_Exception(sprintf('Default error messages for rule %s not defined', $rule));

		//TODO: what to do with nested arrays?
		// array_merge doesn't works correctly
		foreach ((array)$this->messages[$rule] as $code => $message) {
			if (!isset($messages[$code]))
				$messages[$code] = $message;
		}

		if (!$errorCode)
			return $messages[0];

		if (isset($messages[$errorCode])) {
			return $messages[$errorCode];
		}
		throw new BL_Exception(sprintf('Default error messages for rule %s and errorCode %s not defined', $rule, $errorCode));
	}

	protected function formatMessage($message, array $data = array())
	{
		foreach ($data as $key => $value) {
			$data['%' . $key . '%'] = $value;
			unset($data[$key]);
		}
		
		return strtr($message, $data);
	}
}
