<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Validator_Base extends BL_Validator_Abstract
{
	protected $messages = array(
		'numeric' => 'Value of field %field% is not numeric',
		'integer' => 'Value of field %field% is not integer',
		'string'  => 'Value of field %field% is not string',
		'confirm' => 'Value of field %field% not equal with value of %efield%'
	);

    public function getRules()
	{
		return array('num', 'numeric', 'int', 'integer', 'str', 'string', 'confirm', 'equal', 'confirm', 'callback', 'checkbox', 'copy');
	}
	
	public function validate($rule, $field, &$value, &$error, array $messages = array(), array $options = array())
	{
		$params = array('field' => $field, 'value' => $value);
		
		switch ($rule) {
			case 'num':
			case 'numeric':
				if (!is_numeric($value)) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), $params);
					return false;
				}
				break;
			
			case 'int':
			case 'integer':
				if (!$value == (string)ceil($value)) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), $params);
					return false;
				}
				$value = ceil($value);
				break;
			
			case 'str':
			case 'string':
				if (!is_string($value)) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), $params);
					return false;
				}
				break;
			
			case 'confirm':
			case 'equal':
				$altField = $this->getOption($options, 'field');				
				$altValue = $this->validator->getValue($altField);

				if ($value != $altValue) {
					$params['efield'] = $altField;
					$params['evalue'] = $altValue;
					$error = $this->formatMessage($this->getMessage($rule, $messages), $params);
					return false;
				}
				break;
			
			case 'callback':
				$callback = $this->getOption($options, 'callback');
				if (!is_callable($callback)) {
					throw new BL_Exception('Unable to exec callback');
				}
				if (!call_user_func_array($callback, array($field, &$value, &$error, $messages, $options))) {
					$error = $this->formatMessage($error, $params);
					return false;
				}
				break;

				case 'checkbox':
					$value = $value ? 1 : 0;
					break;

				case 'copy':
					$field = $this->getOption($options, 'field');
					$value = $this->validator->getValue($field);
					break;
		}
		
		return true;
	}
}
