<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package   Bicycle Libraries
 * @author    Konstantin.Myakshin
 * @copyright Copyright (c) 2009-2010, Brouzie, Inc.
 * @license   LGPL
 * @link      http://brouzie.com/projects/bicycle-libraries/
 * @link      http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

class BL_Validator_Strings extends BL_Validator_Abstract
{
	const REGEX_EMAIL = '/^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i';
    
	protected $messages = array(
		'minLength' => 'Too long',
		'maxLength' => 'Too short',
		'length'    => array(
			'limits' => 'Length of %field% need to be between %minLength% and %maxLength%',
			'equal'  => 'Length of %field% <> %length%'
		),
		'mail'      => '%field% is not valid email',
		'ip'        => '%field% is not ip address',
		'regexp'    => 'Error: pattern %pattern% hasn\'t mached in %value%',
	);

	public function getRules()
	{
		return array('minLength', 'maxLength', 'length', 'mail', 'ip', 'regexp');
	}
	
	public function validate($rule, $field, &$value, &$error, array $messages = array(), array $options = array())
	{
		$params = array('field' => $field, 'value' => $value);
		
		switch ($rule) {
			case 'minLength':
				$minLength = $this->getOption($options, 'length');
				$length = strlen($value);
				if ($length < $minLength) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), array('field' => $field, 'value' => $value, 'length' => $length, 'minLength' => $minLength));
					return false;
				}
				break;
			
			case 'maxLength':
				$maxLength = $this->getOption($options, 'length');
				$length = strlen($value);
				if ($length > $maxLength) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), array('field' => $field, 'value' => $value, 'length' => $length, 'maxLength' => $maxLength));
					return false;
				}
				break;
			
			case 'length':
				$defaultOptions = array(0 => false, 1 => false);
				$minLength = $this->getOption($options, 0, -1, $defaultOptions);
				$maxLength = $this->getOption($options, 1, -1, $defaultOptions);
				$length = strlen($value);
				
				if ($minLength && $maxLength) {
					if ($length < $minLength || $length > $maxLength) {
						$error = $this->formatMessage($this->getMessage($rule, $messages, 'limits'), array('field' => $field, 'value' => $value, 'length' => $length, 'minLength' => $minLength, 'maxLength' => $maxLength));
						return false;
					}
				} else {
					if ($length != $minLength) {
						$error = $this->formatMessage($this->getMessage($rule, $messages, 'equal'), array('field' => $field, 'value' => $value, 'length' => $length, 'eLength' => $minLength));
						return false;
					}
				}
				break;
			
			case 'mail':
				if (preg_match(self::REGEX_EMAIL, $value) === 0) {
				    $error = $this->formatMessage($this->getMessage($rule, $messages), array('field' => $field, 'value' => $value));
				    return false;
				}
				break;
			
			case 'ip':
				$pattern = '/^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]).){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/';
				if (!preg_match($pattern, $value)) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), array('field' => $field, 'value' => $value));
					return false;
				}
				break;
			
			case 'regexp':
				$pattern = $this->getOption($options, 'pattern');
				if (!preg_match($pattern, $value)) {
					$error = $this->formatMessage($this->getMessage($rule, $messages), array('field' => $field, 'value' => $value, 'length' => $length, 'eLength' => $eLength));
					return false;
				}
				break;
		}
		
		return true;
	}
}
