<?php
/**
 * Bicycle Libraries
 *
 * Set of libraries wich can help you
 *
 * @package		Bicycle Libraries
 * @author		Konstantin.Myakshin
 * @copyright	Copyright (c) 2009-2010, Brouzie, Inc.
 * @license		LGPL
 * @link		http://brouzie.com/projects/bicycle-libraries/
 * @link		http://code.google.com/p/bicycle-libraries/
 * @filesource
 */

/**
 * anyInArray()
 * 
 * @param array $needles
 * @param array $haystack
 * @param bool $strict
 * @return bool
 */
function anyInArray($needles, $haystack, $strict = false)
{
	foreach ($needles as $needle)
		if (in_array($needle, $haystack, $strict))
			return true;
	
	return false;
}

function arrayCut(&$array, $keys)
{
	$return = array();
	$keys = (array)$keys;

	foreach ($array as $key => $value) {
		if (in_array($key, $keys)) {
			$return[$key] = $value;
			unset($array[$key]);
		}
	}

	return 1 == count($keys) ? array_shift($return) : $return;
}

function array_merge_recursive_distinct()
{
	$arrays = func_get_args();
	$base = array_shift($arrays);
	
	if (!is_array($base)) $base = empty($base) ? array() : array($base);

	foreach ($arrays as $append) {
		if (!is_array($append)) $append = array($append);
		foreach ($append as $key => $value) {
			if (!array_key_exists($key, $base) and!is_numeric($key)) {
				$base[$key] = $append[$key];
				continue;
			}
			if (is_array($value) or is_array($base[$key])) {
				$base[$key] = array_merge_recursive_distinct($base[$key], $append[$key]);
			} else if (is_numeric($key)) {
				if (!in_array($value, $base)) $base[] = $value;
			} else {
				$base[$key] = $value;
			}
		}
	}
	return $base;
}

function between($x, $min, $max)
{
	return min(max($min, $x), $max);
}

function isBetween($num, $min, $max)
{
	return ($num <= $max && $num >= $min);
}

if (!function_exists('get_called_class')) {
	function get_called_class()
	{
		$bt = debug_backtrace();
		$l = 0;
		do {
			$l++;
			$lines = file($bt[$l]['file']);
			$callerLine = $lines[$bt[$l]['line'] - 1];
			preg_match('/([a-zA-Z0-9\_]+)::' . $bt[$l]['function'] . '/',
				$callerLine,
				$matches);
			
			if ($matches[1] == 'self') {
				$line = $bt[$l]['line'] - 1;
				while ($line > 0 && strpos($lines[$line], 'class') === false) {
					$line--;
				}
				preg_match('/class[\s]+(.+?)[\s]+/si', $lines[$line], $matches);
			}
		
		} while ($matches[1] == 'parent'  && $matches[1]);
		
		return $matches[1];
	}
}

function iconvArray($array, $from = 'UTF-8', $to = BL_APP_CHARSET)
{
	foreach ($array as &$value) {
		if (is_array($value)) {
			$value = iconvArray($value, $from, $to);
		} elseif (is_string($value)) {
			if (!$from)
				$from = mb_detect_encoding($from);
			$value = iconv($from, $to, $value);
		}
	}
	return $array;
}

function arrayFilter(&$data, $callback = 'stripslashes', array $args = array())
{
	if (is_array($data)) {
		foreach ($data as &$value) {
			$args[0] = $value;
			$value = is_array($value) ? arrayFilter($value, $callback, $args) : call_user_func_array($callback, $args);
		}
	} else {
		$args[0] = $data;
		$data = call_user_func_array($callback, $args);
	}
	return $data;
}

function patternToRegexp($pattern)
{
	static $replacement = array(
		'*'      => '(.+?)',
		':any'   => '(.+?)',
		':num'   => '(\d+?)',
		':day'   => '',
		':month' => '',
		':year'  => '',
	);
	
	$pattern = str_replace('.', '\.', $pattern);
	return strtr($pattern, $replacement);
}

function synonymEncoding($name)
{
	$array = array(
		'windows-1251' => 'cp1251',
		'UTF-8'        => 'utf8'
	);
	
	return isset($array[$name]) ? $array[$name] : NULL;
}

function base64UrlEncode($input = NULL)
{
	if (!$input)
		$input = $_SERVER['REQUEST_URI'];
	return strtr(base64_encode($input), '+/=', '-_,');
}

function base64UrlDecode($input)
{
	return base64_decode(strtr($input, '-_,', '+/='));
}

function json($a = false)
{
	if (NULL === $a)
		return 'null';
	if (false === $a)
		return 'false';
	if (true === $a)
		return 'true';

	if (is_scalar($a)) {
		if (is_float($a)) {
			// always use '.' for floats
			return floatval(str_replace(',', '.', strval($a)));
		}

		if (is_string($a)) {
			static $jsonReplaces = array(array("\\", "/", "\n", "\t", "\r", "\b", "\f", '"'),
				array('\\\\', '\\/', '\\n', '\\t', '\\r', '\\b', '\\f', '\"'));
			return '"' . str_replace($jsonReplaces[0], $jsonReplaces[1], $a) . '"';
		} else
			return $a;
	}
	$isList = true;
	//$is_obj = isset($obj[count($obj) - 1]) ? false : true;
	for ($i = 0, reset($a); $i < count($a); $i++, next($a)) {
		if (key($a) !== $i) {
			$isList = false;
			break;
		}
	}
	$result = array();
	if ($isList) {
		foreach ($a as $v)
			$result[] = json($v);
		
		return '[' . implode(',', $result) . ']';
	
	} else {
		foreach ($a as $k => $v)
			$result[] = json($k) . ':' . json($v);
		
		return '{' . implode(',', $result) . '}';
	}
}

/**
 * template()
 *
 * @param string|null $template
 * @param string|null $ext
 * @param string|null $dir
 * @return BL_Template
 */
function template($template = NULL, $ext = NULL, $dir = NULL)
{
	$class = new BL_Template();
	if (NULL !== $dir)
		$class->baseDir($dir);

	if (!$template)
		return $class;

	return $ext ? $class->load($template, $ext) : $class->load($template);
}
/**
 * templateString()
 *
 * @param string $data
 * @return BL_Template
 */
function templateString($data)
{
	$class = new BL_Template();
	return $class->loadData($data);
}

